import models
import ipaddress
from datetime import datetime
from os import path
from sqlalchemy import (
    create_engine,
    or_,
)
from sqlalchemy.orm import Session


class DatabaseConnector:
    def __init__(self, data_path, in_memory=False):
        if in_memory:
            self.engine = create_engine(
                "sqlite+pysqlite:///:memory:", echo=True, future=True
            )
        else:
            db_path = path.join(data_path, "overseer.sqlite")
            self.engine = create_engine(
                "sqlite+pysqlite:///%s" % db_path,
                echo=True,
                future=True,
            )
        models.Base.metadata.create_all(self.engine)

    def create_scan_result(self, ip_addr, scan_results, hostname=None):
        int_ip_addr = int(ipaddress.ip_addr(ip_addr))
        session = Session(bind=self.engine)

        # Does an existing target exist?
        scan_target = (
            session.query(models.ScanTarget)
            .filter(
                or_(
                    models.ScanTarget.ip == int_ip_addr,
                    models.ScanTarget.hostname == hostname,
                )
            )
            .first()
        )

        # TODO: Do we need to update hostname?

        # Nope, create one
        if not scan_target:
            scan_target = models.ScanTarget(ip=int_ip_addr, hostname=hostname)
            session.add(scan_target)
            session.commit()

        # Create scan history
        scan_history = models.ScanHistory(
            target_id=scan_target.id,
            results=",".join(map(str, scan_results)),
            datetime=datetime.now(),
        )
        session.add(scan_history)
        session.commit()

        session.close()

    def get_scan_results(self, **kwargs):
        """Returns scan results for the queries target.

        Keyword arguments:
        hostname -- The hostname of the target.
        ip_addr -- The IP address of the target.
        """
        if len(kwargs.keys() & {"ip_addr", "hostname"}) != 1:
            raise ValueError("Missing keyword argument: ip_addr or hostname")

        hostname = kwargs["hostname"] if "hostname" in kwargs else None
        ip_addr = kwargs["ip_addr"] if "ip_addr" in kwargs else None
        int_ip_addr = int(ipaddress.ip_addr(ip_addr)) if ip_addr else None

        session = Session(bind=self.engine)

        # Get all scan histories
        scan_histories = (
            session.query(models.ScanHistory)
            .join(models.ScanHistory.target)
            .filter(
                or_(
                    models.ScanTarget.ip == int_ip_addr,
                    models.ScanTarget.hostname == hostname,
                )
            )
            .all()
        )

        session.close()

        return scan_histories


# FOR TESTING PURPOSES
def main():
    data_path = "/Users/evanreichard/Development/git/overseer/src/overseer"
    db = DatabaseConnector(data_path)
    db.create_scan_result("1.2.3.4", [5, 6, 7, 8], "test222.com")
    db.get_scan_results(ip_addr="1.2.3.4")


if __name__ == "__main__":
    main()
